/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.data;

import java.awt.*;
import javax.swing.*;
import tablet.*;

/**
 * <p>Simple viewing algorithm of captured data</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class SimpleTabletDataViewer
    extends TabletDataViewer {
  /**
   * active tablet measures
   */
  private TabletInfo ti;
  /**
   * number of pixels between active area and border
   */
  private int border = 4;
  /**
   * zoom
   */
  private boolean detail = false;
  private boolean showUp;

  /**
   * creates new data viewer for specified panel
   * @param screen panel to draw on
   * @param tabletData data to draw
   */
  public SimpleTabletDataViewer(JPanel screen, TabletData tabletData, boolean showUp) {
    super(screen, tabletData);
    this.showUp = showUp;
    ti = tabletData.getTabletInfo();
  }

  /**
   * painting procedure
   * <p>Method paints data to canvas specified by g param. It preserves aspect
   * ratio of x and y axis. When detail is set to true, it zoom the drawing across
   * entire viewport, still preserve aspect ratio.</p>
   * @param g where to draw
   */
  public void paint(Graphics g) {
    // get measures
    int scr_width = screen.getWidth() - 2 * border;
    int scr_height = screen.getHeight() - 2 * border;
    int tab_width = ti.getMaxX() - ti.getMinX();
    int tab_height = ti.getMaxY() - ti.getMinY();

    // set drawing from top left corner
    int offset_x = 0;
    int offset_y = 0;
    int offset = 0;

    if (detail) { // user want to zoom
      int max_x = Integer.MIN_VALUE, max_y = Integer.MIN_VALUE;
      int min_x = Integer.MAX_VALUE, min_y = Integer.MAX_VALUE;
      // get maximal and minimal coordinates of x and y axis
      for (int i = 0; i < tabletData.size(); i++) {
        TabletPoint point = tabletData.get(i);
        max_x = Math.max(max_x, point.getX());
        max_y = Math.max(max_y, point.getY());
        min_x = Math.min(min_x, point.getX());
        min_y = Math.min(min_y, point.getY());
      }

      tab_width = max_x - min_x + 1;
      tab_height = max_y - min_y + 1;

      // create offset from gray area
      offset = (int) Math.min(tab_width * 0.05, tab_height * 0.05);
      tab_height += offset;
      tab_width += offset;

      // center on screen
      offset_x = -min_x;
      offset_y = -min_y;
      offset /= 2;
    }

    // which dimension have bigger compress ratio
    float r_w = ( (float) scr_width) / (float) tab_width;
    float r_h = ( (float) scr_height) / (float) tab_height;
    float koef = (r_h > r_w) ? r_w : r_h;

    // dimension of active area
    int pic_width = (int) (tab_width * koef);
    int pic_height = (int) (tab_height * koef);

    // top left corner of active area
    int left = (scr_width - pic_width) / 2 + border;
    int top = (scr_height - pic_height) / 2 + border;

    // clear window
    g.setColor(Color.LIGHT_GRAY);
    g.fillRect(0, 0, screen.getWidth(), screen.getHeight());
    // clear active area
    g.setColor(Color.WHITE);
    g.fillRect(left, top, pic_width, pic_height);

    // initialize for drawing
    int x, ox = 0;
    int y, oy = 0;
    boolean pen_down = false;

    g.setColor(Color.black);

    scr_height = screen.getHeight() - top - (int) (offset * koef);
    left += (int) (offset * koef);
    // draw data
    for (int i = 0; i < tabletData.size(); i++) {
      TabletPoint point = tabletData.get(i);
      x = (int) ( (point.getX() + offset_x) * koef);
      y = (int) ( (point.getY() + offset_y) * koef);
      x = x + left;
      y = scr_height - y;

      if (!point.isTouched() && showUp) {
        pen_down = false;
        continue;
      }

      if (!pen_down) { // beginning of new line
        pen_down = true;
        ox = x;
        oy = y;
      }

      g.drawLine(x, y, ox, oy);
      ox = x;
      oy = y;
    }
  }

  /**
   * zoom or unzoom view
   */
  public void setDetail(boolean detail) {
    this.detail = detail;
    screen.repaint();
  }

  /**
   * is view zoomed?
   */
  public boolean getDetail() {
    return detail;
  }

  public void setShowUp(boolean showUp) {
    this.showUp = showUp;
  }

  /**
   * sets new tablet measures
   */
  public void setTabletInfo(TabletInfo tabletInfo) {
    this.ti = tabletInfo;
  }
}
