/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.data;

import tablet.*;
import java.util.*;
import java.io.*;
import java.util.zip.*;

/**
 * <p>Class stores list of tablet points creating drawing.</p>
 * <p>Data are stored in zip archive with one file in it. On name of inner file
 * is taken no respect.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class TabletData {
  /**
   * list of TabletPoints
   */
  private List<TabletPoint> vektor = new ArrayList<TabletPoint>();
  /**
   * tablet measures with which were data captured
   */
  private TabletInfo ti;

  /**
   * creates new empty list with specified tablet
   * @param tabletInfo tablet measures
   */
  public TabletData(TabletInfo tabletInfo) {
    this.ti = tabletInfo;
  }

  /**
   * loads list from file
   * <p>Method reads file which is supposed to be zip archive. Archive contains
   * one file, no matter of its name. Inner file consists of blocks of 20 bytes
   * representing one tablet point.</p>
   * @param file file to load from
   * @param tabletInfo tablet measures
   */
  public TabletData(File file, TabletInfo tabletInfo) {
    this.ti = tabletInfo;
    ZipInputStream in = null;
    try {
      in = new ZipInputStream(new FileInputStream(file));
      in.getNextEntry(); // first and only entry in zip file
      byte buff[] = new byte[20];
      int len = 0;
      while((len = in.read(buff)) > -1) {
        while (len != 20) { // read until all twenty bytes of one point are collected
          byte array[] = new byte[20-len];
          len += in.read(array);
          System.arraycopy(array, 0, buff, 20-array.length, array.length);
        }
        this.add(new TabletPoint(buff));
      }
      in.closeEntry();
    } catch(IOException e) {
      System.err.println("Chyba nacitani dat");
      e.printStackTrace();
    } finally {
      if (in != null) {
        try {
          in.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * appends TabletPoint to list
   */
  public void add(TabletPoint tabletPoint) {
    vektor.add(tabletPoint);
  }

  /**
   * retrieves TabletPoint at specified position
   */
  public TabletPoint get(int index) {
    return vektor.get(index);
  }

  /**
   * returns length of list of points
   */
  public int size() {
    return vektor.size();
  }

  /**
   * saves list to file
   * <p>Method save sequence of tablet points to file. File is zip archive and
   * contains one file with main data. Data are blocks of 20 bytes each block
   * represent one tablet point.</p>
   * @return boolean whether was save succesful
   */
  public boolean save(File file) {
    ZipOutputStream out = null;
    try {
      out = new ZipOutputStream(new FileOutputStream(file));
      out.setMethod(ZipOutputStream.DEFLATED);
      out.setLevel(6);
      String name = file.getName().substring(0, file.getName().indexOf(".zip")) + ".dat";
      ZipEntry anEntry = new ZipEntry(name);
      out.putNextEntry(anEntry);
      for(TabletPoint tp: vektor) {
        tp.print(out);
      }
    } catch(IOException e) {
      System.err.println("Nepodarilo se zapsat data");
      e.printStackTrace();
      return false;
    } finally {
      if(out != null) {
        try {
          out.close();
        } catch (IOException e) {}
      }
    }
    return true;
  }

  public void trim() {
    for (int i = vektor.size() - 1; i >= 0; i--) {
      if (!vektor.get(i).isTouched()) {
        vektor.remove(i);
      }
      else {
        return;
      }
    }
  }

  /**
   * returns information about tablet with which were data captured
   */
  public TabletInfo getTabletInfo() {
    return ti;
  }
}
