/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import tablet.*;
import java.text.*;
import java.util.Date;
import java.io.*;
import org.jdom.*;

/**
 * <p>Class contains and information about tablet and about captured data.</p>
 * <p>Data itself are in file with name: <em>pacient-hash</em>_YYYY_MM_DD_HH_MM_SS.zip</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @see spirals.data.TabletData TabletData
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class Data {
  /**
   * pacient whom data belongs to
   */
  private Visit visit;
  /**
   * type of drawing
   */
  private String description;
  /**
   * creation time
   */
  private Date date;
  /**
   * file with data
   */
  private File file;
  /**
   * data comment
   */
  private String memo;
  /**
   * tablet measures
   */
  private TabletInfo ti;

  /**
   * name of data tag
   */
  public static final String dataTag = "data";
  /**
   * name of data type attribute
   */
  public static final String descAttr = "desc";
  /**
   * name of tablet tag
   */
  public static final String tabletTag = "tablet";
  /**
   * name of tablet name attribute
   */
  public static final String nameAttr = "name";
  /**
   * name of tabet's minimal x coordinate attribute
   */
  public static final String minxAttr = "minx";
  /**
   * name of tabet's maximal x coordinate attribute
   */
  public static final String maxxAttr = "maxx";
  /**
   * name of tabet's minimal y coordinate attribute
   */
  public static final String minyAttr = "miny";
  /**
   * name of tabet's maximal y coordinate attribute
   */
  public static final String maxyAttr = "maxy";
  /**
   * name data file attribute
   */
  public static final String fileAttr = "file";
  /**
   * name of comment tag
   */
  public static final String memoTag = "memo";

  /**
   * format for naming file
   */
  public static final String dateFormat = "yyyy_MM_dd_HH_mm_ss";

  /**
   * create new TabletInfo from XML tag
   */
  private static final TabletInfo getTabletInfoFromTag(Element tabEl) {
    int minx = Integer.parseInt(tabEl.getAttributeValue(minxAttr));
    int maxx = Integer.parseInt(tabEl.getAttributeValue(maxxAttr));
    int miny = Integer.parseInt(tabEl.getAttributeValue(minyAttr));
    int maxy = Integer.parseInt(tabEl.getAttributeValue(maxyAttr));
    return new TabletInfo(tabEl.getAttributeValue(nameAttr), minx, maxx, miny,
                          maxy);
  }

  /**
   * crete new tablet data for pacient from XML tag
   */
  public Data(Visit visit, Element data) {
    this(visit, getTabletInfoFromTag(data.getChild(tabletTag)),
         data.getAttributeValue(descAttr), data.getAttributeValue(fileAttr),
         data.getChildTextTrim(memoTag));
  }

  /**
   * create new tablet data for pacient from specified information
   * <p>From file name is extracted date and created relative path to this file
   * from database root.</p>
   * @param pacient Pacient whom belong data
   * @param tabletInfo tablet measures
   * @param description type of drawing
   * @param file name of file
   * @param memo comment
   */
  public Data(Visit visit, TabletInfo tabletInfo, String description,
              String file, String memo) {
    this.visit = visit;
    this.description = description;
    this.memo = memo;
    this.ti = tabletInfo;
    // read date of creation from file name
    SimpleDateFormat df = new SimpleDateFormat("'" +
                                               visit.getPacient().getHashName() +
                                               "'_" + dateFormat);
    try {
      this.date = df.parse(file);
    }
    catch (ParseException e) {
      System.err.println("Chyba parsovani data");
    }
    // relative path to file from application root
    this.file = new File(visit.getPacient().getDatabase().getEngine().
                         getConfiguration().getDatabaseRoot(),
                         visit.getPacient().getHashName() + File.separator +
                         file);
  }

  /**
   * create XML element for saving
   * <p>Element contains type of drawing, file name with data, comment of drawing,
   * and tablet measures with which were data captured.</p>
   */
  public Element getElement() {
    Element dataEl = new Element(dataTag);
    dataEl.setAttribute(descAttr, getDescription());
    dataEl.setAttribute(fileAttr, toFileName());
    Element tabEl = new Element(tabletTag);
    TabletInfo ti = getTabletInfo();
    tabEl.setAttribute(nameAttr, ti.getTabletName());
    tabEl.setAttribute(minxAttr, "" + ti.getMinX());
    tabEl.setAttribute(maxxAttr, "" + ti.getMaxX());
    tabEl.setAttribute(minyAttr, "" + ti.getMinY());
    tabEl.setAttribute(maxyAttr, "" + ti.getMaxY());
    Element memoEl = new Element(memoTag);
    memoEl.setText(getMemo());
    dataEl.addContent(memoEl);
    dataEl.addContent(tabEl);
    return dataEl;
  }

  /**
   * return type of drawing
   */
  public String getDescription() {
    return description;
  }

  /**
   * return type of drawing and date of creation
   */
  public String toString() {
    SimpleDateFormat df = new SimpleDateFormat("dd. MM. yyyy, HH:mm:ss");
    return description + " (" + df.format(date) + ")";
  }

  /**
   * return name of file
   */
  public String toFileName() {
    SimpleDateFormat df = new SimpleDateFormat(dateFormat);
    return visit.getPacient().getHashName() + "_" + df.format(date) + ".zip";
  }

  /**
   * return file with relative path from application root
   */
  public File getFile() {
    return file;
  }

  /**
   * return data comment
   */
  public String getMemo() {
    return memo;
  }

  /**
   * returns tablet measures
   */
  public TabletInfo getTabletInfo() {
    return ti;
  }

  /**
   * set new tablet measures
   */
  public void setTabletInfo(TabletInfo tabletInfo) {
    ti = tabletInfo;
  }

  /**
   * set new comment
   */
  public void setMemo(String memo) {
    this.memo = memo;
  }

  /**
   * set new type of drawing
   */
  public void setDescription(String description) {
    visit.getTestList().updateHist(this.description, description);
    this.description = description;
  }
}
