/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-09-21
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.database;

import java.util.*;
import java.io.*;
import javax.swing.JOptionPane;
import org.jdom.*;
import java.util.zip.*;
import javax.swing.JFileChooser;

/**
 * <p>Root class for storing database.</p>
 * <p>Each time database is exported all pacients are stored in random order without
 * surname and firstname. No corespondence between database and exported data should
 * be found.</p>
 * <p>Database contains Pacients. Pacients have Visits and Visits consist of Data.
 * Database is stored in directory 'data' containing root file and direcotries of pacients.
 * Only root file is loaded on application startup. Each pacient directory contains
 * xml files representing visits and zip archives with captured tablet data for visits.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @see spirals.data.TabletData TabletData
 * @see Data Data
 * @see Pacient Pacient
 * @see Visit Visit
 *
 * @author Jan Dolezel
 * @version 1.1.2
 */
public class Database {
  /**
   * engine
   */
  private Engine engine;
  /**
   * main database content
   */
  private HashMap<String, Pacient> database = null;
  /**
   * sorted list of all pacients
   */
  private List<Pacient> listAll = null;
  /**
   * sorted list of choosen pacients
   */
  private List<Pacient> listRestricted = null;
  /**
   * whether show all or only choosen pacients
   */
  private boolean restricted = false;
  /**
   * current algorithm for sorting
   */
  private SORT howSort = SORT.byName;
  /**
   * root file of stored database
   */
  private File rootFile;
  /**
   * date of last backup
   */
  private Date lastBackup;

  /**
   * database tag (root element)
   */
  public static final String databaseTag = "database";
  /**
   * name of backup date attribute
   */
  public static final String backupAttr = "backup";

  /**
   * create new database or load content from file
   * <p>Check if database exist. If so, load it from disk, otherwise create new one.</p>
   */
  public Database(Engine engine) {
    this.engine = engine;
    rootFile = new File(engine.getConfiguration().getDatabaseRoot(),
                        "database.xml");
    if (rootFile.exists()) {
      this.loadFromFile(rootFile);
    }
    else {
      this.createDB();
    }
    listRestricted = new ArrayList(0);
    this.sort();
  }

  /**
   * create new database
   * <p>Creates new database directory and root file.</p>
   */
  private void createDB() {
    System.err.println("Neni databaze");
    database = new HashMap(10);
    listAll = new ArrayList(10);
    lastBackup = new Date();
    if (engine.getConfiguration().getDatabaseRoot().exists() || engine.getConfiguration().getDatabaseRoot().mkdir()) {
      this.save();
      JOptionPane.showMessageDialog(engine.getMainFrame(),
                                    "Byla zaloena nov databze",
                                    "Vytvaen databze",
                                    JOptionPane.WARNING_MESSAGE);
    }
    else {
      System.err.println("Nelze vytvorit adresar data");
      JOptionPane.showMessageDialog(engine.getMainFrame(),
                                    "Nelze zaloit databzi!",
                                    "Chyba pi vytvaen databze",
                                    JOptionPane.ERROR_MESSAGE);
      System.exit(1);
    }
  }

  /**
   * load database from file
   * <p>Load database root file. All other files ar loaded after first need of them.
   * Main file consist only from pacient names, their born date, right-hand attribute
   * and hash representation. Hash identifies pacient in database and in directory
   * structure on disk.</p>
   */
  private void loadFromFile(File file) {
    try {
      Document doc = engine.getXMLBuilder().build(file);
      Element root = doc.getRootElement();
      // get last backup date
      lastBackup = Engine.getSimpleDate(root.getAttributeValue(backupAttr));
      // load pacients
      List<Element> pacients = root.getChildren(Pacient.pacientTag);
      database = new HashMap<String, Pacient> (pacients.size());
      for (Element e : pacients) {
        Pacient pac = new Pacient(this, e);
        database.put(pac.getHashName(), pac);
      }
      listAll = new ArrayList(database.values());
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * save database to disk
   * <p>Calls getElement() to produce XML representation and store it to disk.</p>
   * @see #getElement()
   */
  public void save() {
    Document doc = new Document(getElement());
    FileOutputStream os = null;
    try {
      os = new FileOutputStream(rootFile);
      engine.getXMLOutputer().output(doc, os);
    }
    catch(IOException e) {
      System.err.println(e);
    } finally {
      if (os != null) {
        try {
          os.close();
        }
        catch (IOException e) {}
      }
    }
  }

  /**
   * return XML representation of database
   * <p>Stores all pacient information.</p>
   */
  public Element getElement() {
    Element root = new Element(databaseTag);
    root.setAttribute(backupAttr,
                      Engine.getSimpleDateFormat().format(lastBackup));
    for (Pacient p : database.values()) {
      root.addContent(p.createElement());
    }
    return root;
  }

  /**
   * return list of all pacients in database
   */
  public List<Pacient> getAll() {
    return listAll;
  }

  /**
   * returns list of choosen pacients
   */
  public List<Pacient> getRestricted() {
    return listRestricted;
  }

  /**
   * clear list with choosen pacients
   */
  public void clearRestricted() {
    listRestricted = new ArrayList();
  }

  /**
   * add pacients to list of choosen which have visit between dateBegin and dateEnd
   */
  public void setRestrictedVisit(Date dateBegin, Date dateEnd) {
    for (Pacient p : database.values()) {
      if (p.getLastVisit() != null) {
        if (p.getLastVisit().getVisitDate().compareTo(dateBegin) >= 0 &&
            p.getLastVisit().getVisitDate().compareTo(dateEnd) <= 0 &&
            !listRestricted.contains(p)) {
          listRestricted.add(p);
        }
      }
    }
  }

  /**
   * add pacients to list of choosen which were born between dateBegin and dateEnd
   */
  public void setRestrictedBorn(Date dateBegin, Date dateEnd) {
    for (Pacient p : database.values()) {
      if (p.getBornDate().compareTo(dateBegin) >= 0 &&
          p.getBornDate().compareTo(dateEnd) <= 0 && !listRestricted.contains(p)) {
        listRestricted.add(p);
      }
    }
  }

  /**
   * add pacients to list of choosen which surname is between nameBegin and nameEnd
   */
  public void setRestrictedName(String nameBegin, String nameEnd) {
    for (Pacient p : database.values()) {
      if (Engine.AbcComparator.getInstance().compare(p.getSurname(), nameBegin) >= 0 &&
          Engine.AbcComparator.getInstance().compare(p.getSurname(), nameEnd) <= 0 &&
          !listRestricted.contains(p)) {
        listRestricted.add(p);
      }
    }
  }

  /**
   * sets all or only choosen pacients
   */
  public void setRestricted(boolean restricted) {
    this.restricted = restricted;
    this.sort();
  }

  /**
   * return size of actual list of pacients
   */
  public int getTreeCount() {
    if (restricted) {
      return listRestricted.size();
    }
    else {
      return listAll.size();
    }
  }

  /**
   * return pacient on specified position in actual list of pacients
   */
  public Pacient getTreePacient(int index) {
    if (restricted) {
      return listRestricted.get(index);
    }
    else {
      return listAll.get(index);
    }
  }

  /**
   * return index of pacient in actual list of pacients
   */
  public int getTreeIndex(Pacient pacient) {
    if (restricted) {
      return listRestricted.indexOf(pacient);
    }
    else {
      return listAll.indexOf(pacient);
    }
  }

  /**
   * sort both lists of pacients
   */
  public void sort() {
    this.sort(howSort);
  }

  /**
   * sort both lists of pacients with specified criteria
   */
  public void sort(SORT how) {
    this.howSort = how;
    switch (how) {
      case byName:
        this.sort(Engine.nameComparator);
        break;
      case byBornDate:
        this.sort(Engine.bornComparator);
        break;
      case byLastVisit:
        this.sort(Engine.visitComparator);
        break;
    }
    if (engine.getMainFrame().getTree() != null) {
      engine.getMainFrame().getTree().getTree().updateUI();
    }
  }

  /**
   * sort both lists of pacients with specified comparator
   */
  private void sort(Comparator c) {
    Collections.sort(listAll, c);
    Collections.sort(listRestricted, c);
  }

  /**
   * generates next unused hash string for new pacient
   * <p>Output is something that nears md5 hash output.</p>
   */
  public String getNextID() {
    String str;
    do {
      str = Engine.hash(Engine.hash(String.format("%x",
                                                  (long) (Math.random() *
          Long.MAX_VALUE))));
    }
    while (database.containsValue(str));
    return str;
  }

  /**
   * add pacient to database and save
   * <p>Updates or inserts information about pacient, sort lists of pacients and saves database to disk.</p>
   */
  public void addPacient(Pacient pacient) {
    String newID = pacient.getHashName();
    if (!database.containsKey(newID)) {
      database.put(newID, pacient);
      listAll.add(pacient);
    }
    this.save();
    this.sort();
//    engine.getMainFrame().getTree().getTree().updateUI();
  }

  /**
   * return number of all pacients
   */
  public long getPacientCount() {
    return database.size();
  }

  /**
   * return size of database stored on disk
   * @return long number of bytes
   */
  public long getDatabaseSize() {
    long size = 0;
    for (File file : engine.getConfiguration().getDatabaseRoot().listFiles()) {
      if (file.isDirectory()) {
        for (File f : file.listFiles()) {
          size += f.length();
        }
      }
      else {
        size += file.length();
      }
    }
    return size;
  }

  /**
   * return date of last database backup
   */
  public Date getLastBackup() {
    return lastBackup;
  }

  /**
   * return engine
   */
  public Engine getEngine() {
    return engine;
  }

  /**
   * must be called when database is backuped
   */
  public void backuped() {
    lastBackup = new Date();
    this.save();
  }

  /**
   * exports data from database to user choosen file
   * <p>All database data are read, new hash string for pacients are generated.
   * These 'hash' strings are from numbered sequence form zero to number of pacients.
   * Each pacient recieves random number from this sequence. In each export the same
   * pacient will have another number from sequence. The links in file structure
   * based on these hash strings must be updated.<br>
   * Exported database is saved in zip archive.</p>
   */
  public void export() {
    JFileChooser fc = new JFileChooser();
    fc.setSelectedFile(new File("export.zip"));
    if (fc.showSaveDialog(engine.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
      // user selected file
      File outFile = fc.getSelectedFile();
      ZipOutputStream out = null;
      try {
        out = new ZipOutputStream(new FileOutputStream(outFile));
        out.setMethod(ZipOutputStream.DEFLATED);
        out.setLevel(6);

        // create root element of database
        Element root = new Element(databaseTag);
        root.setAttribute(backupAttr,
                          Engine.getSimpleDateFormat().format(new Date()));
        long newID = 1;
        Collections.shuffle(listAll);
        // export pacients
        for (Pacient p : listAll) {
          // name pacients in sequence begining 00000001
          Element pacient = p.createElement(makeID(newID), "",
                                            "" + makeID(newID), p.isRightHanded(),
                                            p.getBornDate());
          // for all visits of pacient
          for (Visit v : p.getVisits()) {
            Document doc = engine.getXMLBuilder().build(v.getFile());
            Element visRoot = doc.getRootElement();
            // get all data and rename it
            List<Element> list = visRoot.getChildren(Data.dataTag);
            for (Element e : list) {
              String file = e.getAttributeValue(Data.fileAttr);
              e.setAttribute(Data.fileAttr,
                             makeID(newID) + file.substring(file.indexOf('_')));
            }
            // rename visit file
            String oldName = v.getFile().getName();
            String name = makeID(newID) + File.separator + makeID(newID) +
                oldName.substring(oldName.indexOf('_'));
            out.putNextEntry(new ZipEntry(name));
            // copy visit
            engine.getXMLOutputer().output(doc, out);
            // for all data in visit
            for (Data d : v.getData()) {
              // rename file
              String dname = d.toFileName();
              dname = makeID(newID) + File.separator + makeID(newID) +
                  dname.substring(dname.indexOf('_'));
              out.putNextEntry(new ZipEntry(dname));
              // copy data
              Engine.copy(new FileInputStream(d.getFile()), out);
            }
          }
          root.addContent(pacient);
          newID++;
        }

        // copy root file
        Document doc = new Document(root);
        out.putNextEntry(new ZipEntry("database.xml"));
        engine.getXMLOutputer().output(doc, out);
        System.err.println("Databaze exportovana");
        JOptionPane.showMessageDialog(engine.getMainFrame(),
                                      "Export databze probhl",
                                      "Export databze",
                                      JOptionPane.INFORMATION_MESSAGE);
      }
      catch (IOException e) {
        System.err.println("Nelze exportovat databazi");
      }
      catch (JDOMException e) {
        System.err.println("Chyba parsovani souboru databaze.");
        System.err.println(e.getMessage());
      }
      finally {
        if (out != null) {
          try {
            out.close();
          }
          catch (Exception e) {
            System.err.println("Chyba pri zavirani souboru");
          }
        }
      }
    }
    this.sort();
  }

  /**
   * create text representation of long
   * @return String 8 characters
   */
  private String makeID(long id) {
    return String.format("%08d", id);
  }

  /**
   * <p>Enumerates sorting possibilities</p>
   */
  public static enum SORT {
    /**
     * sort ascending by pacient name
     */
    byName,
    /**
      * sort ascending by pacient born date
      */
    byBornDate,
    /**
     * sort descending by pacient's lat visit
     */
    byLastVisit} ;
    }
