/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-09-12
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.database;

import spirals.motol.gui.*;
import tablet.*;
import javax.swing.*;
import java.io.*;
import java.util.zip.*;
import org.jdom.output.XMLOutputter;
import org.jdom.input.SAXBuilder;
import java.text.*;
import java.util.*;

/**
 * <p>Class with static and useful utilities used across application.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
public class Engine {
  /**
   * main frame
   */
  private MainFrame mf;
  /**
   * active tablet
   */
  private Tablet tablet;
  /**
   * database
   */
  private Database database;
  /**
   * confihuration
   */
  private Configuration config;
  /**
   * date formater: DD. MM. YYYY
   */
  private static final SimpleDateFormat sdf = new SimpleDateFormat(
      "dd. MM. yyyy");

  /**
   * starts application with main frame and configuration
   * <p>Load database, then tries to open tablet.</p>
   */
  public Engine(MainFrame frame, Configuration conf) {
    this.mf = frame;
    this.config = conf;
    this.database = new Database(this);
    try {
      this.tablet = Tablet.getInstance(config.getAvailableTablets().get(config.getSelectedTabletIndex()), config.getTimeout());
      tablet.open();
    }
    catch (Error e) {
//      e.printStackTrace();
      System.err.println("Chyba inicializace tabletu");
      JOptionPane.showMessageDialog(mf,
                                    "Tablet nelze nalzt!",
                                    "Chyba",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /**
   * return main frame
   */
  public MainFrame getMainFrame() {
    return mf;
  }

  /**
   * return active tablet
   */
  public Tablet getTablet() {
    return tablet;
  }

  /**
   * return database
   */
  public Database getDatabase() {
    return database;
  }

  /**
   * set database to view all or only choosen pacients
   */
  public void setRestricted(boolean restricted) {
    database.setRestricted(restricted);
  }

  /**
   * return application configuration
   */
  public Configuration getConfiguration() {
    return config;
  }

  /**
   * set tablet as active tablet (if tablet is among available tablets)
   */
  public boolean selectTablet(TabletInfo ti) {
    if (config.getAvailableTablets().contains(ti)) {
      tablet.setTabletInfo(ti);
      config.setSelectedTabletIndex(config.getAvailableTablets().indexOf(ti));
      return true;
    }
    return false;
  }

  private static final int begin = 0xcAcAcAcA;
  private static final int end = 0x35353535;
  /**
   * hashes string to 8 characters
   * <p>Reminds first pass of md5.</p>
   */
  public static final String hash(String str) {
    long val = Long.valueOf(str, 16);
    int v1 = (int) (val % Integer.MAX_VALUE);
    int v2 = (int) (val / Integer.MAX_VALUE);
    int r1 = v1 ^ begin;
    int r2 = v2 ^ begin;
    int r3 = v1 ^ end;
    int r4 = v2 ^ end;
    int r5 = v1 ^ v2;
    double r = r1;
    r += r2 << 1;
    r += r3 << 2;
    r += r4 << 3;
    r += r5 << 4;
    r += r5 << 5;
    r += r2 << 6;
    long res = (long) (Math.abs(r) % Long.MAX_VALUE);
    return Long.toHexString(res);
  }

  /**
   * Recursively archives directory to ZipOutputStream
   * @param zipDir directory/file to archive
   * @param zos archive stream
   * @param saveDir name of parent directory in archive
   * @throws IOException
   */
  public static final void zipDir(File zipDir, ZipOutputStream zos,
                                  File saveDir) throws
      IOException {
    for (File f : zipDir.listFiles()) {
      if (f.isDirectory()) {
        zipDir(f, zos, new File(saveDir, f.getName()));
      }
      else {
        FileInputStream fis = new FileInputStream(f);
        ZipEntry anEntry = new ZipEntry(saveDir + File.separator + f.getName());
        zos.putNextEntry(anEntry);
        copy(fis, zos);
        fis.close();
      }
    }
  }

  /**
   * Recursively copies directory to destination
   * @param copyDir source directory/file
   * @param saveDir destination directory
   * @throws IOException
   */
  public static final void copyDir(File copyDir, File saveDir) throws
      IOException {
    saveDir.mkdir();
    for (File f : copyDir.listFiles()) {
      if (f.isDirectory()) {
        copyDir(f, new File(saveDir, f.getName()));
      }
      else {
        FileInputStream fis = new FileInputStream(f);
        FileOutputStream fos = new FileOutputStream(new File(saveDir, f.getName()));
        copy(fis, fos);
        fis.close();
        fos.close();
      }
    }
  }

  /**
   * copies bytes from input stream to output stream
   * @throws IOException
   */
  public static final void copy(InputStream in, OutputStream out) throws
      IOException {
    byte[] buf = new byte[2156];
    int len;
    while ( (len = in.read(buf)) > 0) {
      out.write(buf, 0, len);
    }
  }

  /**
   * return XML reader
   */
  public SAXBuilder getXMLBuilder() {
    return config.builder;
  }

  /**
   * return XML outputter
   */
  public XMLOutputter getXMLOutputer() {
    return config.serializer;
  }

  /**
   * converts string to date (DD. MM. YYYY)
   * @param date String
   * @return Date
   */
  public static final Date getSimpleDate(String date) {
    Date ret = null;
    try {
      ret = sdf.parse(date);
    }
    catch (ParseException e) {
      System.out.println("Chyba parsovani dat: " + date);
    }
    return ret;
  }

  /**
   * return date formatter (DD. MM. YYYY)
   */
  public static final DateFormat getSimpleDateFormat() {
    return sdf;
  }

  /**
   * comparator of two pacients by name
   */
  static final Comparator nameComparator = new Comparator<Pacient> () {
    public int compare(Pacient p1, Pacient p2) {
      int res = AbcComparator.getInstance().compare(p1.getSurname(), p2.getSurname());
      return (res == 0) ? AbcComparator.getInstance().compare(p1.getFirstname(), p2.getFirstname()) : res;
    }
  };

  /**
   * comparator of two pacients by born date
   */
  static final Comparator bornComparator = new Comparator<Pacient> () {
    public int compare(Pacient p1, Pacient p2) {
      return p1.getBornDate().compareTo(p2.getBornDate());
    }
  };

  /**
   * comparator of two pacients by last visit
   */
  static final Comparator visitComparator = new Comparator<Pacient> () {
    public int compare(Pacient p1, Pacient p2) {
      if ((p1.getLastVisit() == null) && (p2.getLastVisit() == null)) {
        return 0;
      } else if (p1.getLastVisit() == null) {
        return -1;
      } else if (p2.getLastVisit() == null) {
        return 1;
      }
      return p2.getLastVisit().getVisitDate().compareTo(p1.getLastVisit().
          getVisitDate());
    }
  };

  /**
   * czech lexicographical comparator of two strings
   * <p>Sorting of characters with  above should be performed otherwise. These
   * characters are treated as with no sign above them and 'e' added after them.</p>
   */
  public static class AbcComparator implements Comparator<String> {
    private static final char[] translateTableFrom = {'','','','','','','','','','','','','','','','',''};
    private static final char[] translateTableTo   = {'a','a','d','e','e','e','i','n','o','o','s','s','t','u','u','u','y'};
    /**
     * first pass alphabet
     */
    private static final String abcFirst = "abcdefghijklmnopqrstuvwxyz0123456789";
    /**
     * second pass alphabet
     */
    private static final String abcSecond = "abcdefghijklmnopqrstuvwxy\u00FFz0123456789";
    /**
     * position of ch in first alphabet
     */
    private static final int chF = 9;
    /**
     * position of ch in second alphabet
     */
    private static final int chS = 15;

    /**
     * static instance of class
     */
    private static final AbcComparator instance = new AbcComparator();

    /**
     * use AbcComparator.getInstance()
     */
    private AbcComparator() {}

    /**
     * return comparator instance
     */
    public static final AbcComparator getInstance() {
      return instance;
    }

    /**
     * compares two string with two passes<br>
     * first > second => -1<br>
     * first < second =>  1<br>
     * first == second => 0
     */
    public int compare(String str1, String str2) {
      int res = comp(str1, str2, abcFirst, chF);
      return (res == 0) ? comp(str1, str2, abcSecond, chS) : res;
    }

    /**
     * compares two string with specified alphabet
     */
    private int comp(String str1, String str2, String abc, int ch) {
      int len1 = str1.length();
      int len2 = str2.length();
      for (int i = 0; i < len1; i++) {
        if (i >= len2) break;

        char a = Character.toLowerCase(str1.charAt(i));
        char b = Character.toLowerCase(str2.charAt(i));
        // ch in first string?
        if ( (a == 'c') && (i < (len1 - 1)) && (Character.toLowerCase(str1.charAt(i + 1)) == 'h')) {
          if (b == 'c') {
            if ( (i < (len2 - 1)) && (Character.toLowerCase(str2.charAt(i + 1)) == 'h')) {
              continue;
            }
            return 1;
          }
          int pos2 = abc.indexOf(b);
          pos2 = (pos2 < 0) ? abc.indexOf(toChar(b)) : pos2; // used only in first pass
          return (pos2 < ch) ? 1 : -1;
        }
        // ch in second string?
        if ( (b == 'c') && (i < (len2 - 1)) && (Character.toLowerCase(str2.charAt(i + 1)) == 'h')) {
          int pos1 = abc.indexOf(a);
          pos1 = (pos1 < 0) ? abc.indexOf(toChar(a)) : pos1; // used only in first pass
          return (pos1 < ch) ? -1 : 1;
        }

        int pos1 = abc.indexOf(a);
        int pos2 = abc.indexOf(b);
        pos1 = (pos1 < 0) ? abc.indexOf(toChar(a)) : pos1; // used only in first pass
        pos2 = (pos2 < 0) ? abc.indexOf(toChar(b)) : pos2; // used only in first pass
        if (pos1 != pos2) return (pos1 < pos2) ? -1 : 1;
      }
      return (len1 < len2) ? -1 : ( (len1 > len2) ? 1 : 0);
    }

    /**
     * in first pass are converted some characters
     */
    private char toChar(char c) {
      for(int i = 0; i < translateTableFrom.length; i++) {
        if(c == translateTableFrom[i]) return translateTableTo[i];
      }
//    System.out.println("chyba: " + c);
      return c;
    }
  }
}
