/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-09-12
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.database;

import java.io.*;
import java.util.*;
import org.jdom.*;

/**
 * <p>Class representing one pacient.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
public class Pacient {
  /**
   * database where pacient belongs
   */
  private Database database;
  /**
   * list of pacient's visits
   */
  private Visits visits;
  /**
   * hash representation of pacient
   */
  private String hashName;
  /**
   * pacient's firstname
   */
  private String firstname = "";
  /**
   * pacient's surtname
   */
  private String surname = "";
  /**
   * pacient's born date
   */
  private Date born = null;
  /**
   * right-handed pacient
   */
  private boolean rhp = true;
  /**
   * name of pacient tag (root element)
   */
  public static final String pacientTag = "pacient";
  /**
   * name of pacient's id attribute
   */
  public static final String IDAttr = "ID";
  /**
   * name of pacient's firstname attribute
   */
  public static final String firstnameAttr = "firstname";
  /**
   * name of pacient's surname attribute
   */
  public static final String surnameAttr = "surname";
  /**
   * name of pacient's born date attribute
   */
  public static final String bornAttr = "born";
  /**
   * name of pacient's right handed attribute
   */
  public static final String rhpAttr = "rhp";

  /**
   * create new empty pacient in database
   */
  public Pacient(Database database) {
    this(database, database.getNextID(), new Date(), "", "", true);
  }

  /**
   * create new pacient in database from XML element
   */
  public Pacient(Database database, Element pacient) {
    this(database, pacient.getAttributeValue(Pacient.IDAttr),
         Engine.getSimpleDate(pacient.getAttributeValue(Pacient.bornAttr)),
         pacient.getAttributeValue(Pacient.firstnameAttr),
         pacient.getAttributeValue(Pacient.surnameAttr),
         pacient.getAttributeValue(Pacient.rhpAttr).equals("1") ? true : false);
  }

  /**
   * create new pacient in database
   * @param database database
   * @param hashName hash representatio of pacient in database
   * @param born born date
   * @param firstname firstname
   * @param surname surname
   * @param rhp right-handed person
   */
  public Pacient(Database database, String hashName, Date born,
                 String firstname, String surname, boolean rhp) {
    this.database = database;
    this.hashName = hashName;
    this.born = born;
    this.firstname = firstname;
    this.surname = surname;
    this.rhp = rhp;
    this.visits = new Visits(this, database.getEngine());
  }

  /**
   * create XML element from pacient
   */
  public Element createElement() {
    return createElement(getHashName(), getFirstname(), getSurname(), isRightHanded(), getBornDate());
  }

  /**
   * create XML element of pacient from given atributes
   * <p>Pacient have firstname, surname, hash representation, born date and right-hand attribute.</p>
   * @param hashName String hash representation
   * @param firstname String firstname
   * @param surname String surname
   * @param rhp boolean right-handed person
   * @param born Date born date
   * @return Element XML element
   */
  public Element createElement(String hashName, String firstname,
                               String surname, boolean rhp, Date born) {
    Element pacient = new Element(pacientTag);
    pacient.setAttribute(Pacient.IDAttr, hashName);
    pacient.setAttribute(Pacient.firstnameAttr, firstname);
    pacient.setAttribute(Pacient.surnameAttr, surname);
    pacient.setAttribute(Pacient.rhpAttr, rhp ? "1" : "0");
    pacient.setAttribute(Pacient.bornAttr,
                         Engine.getSimpleDateFormat().format(born));
    return pacient;
  }

  /**
   * return number of visits
   */
  public int visitCount() {
    return visits.getVisits().size();
  }

  /**
   * return visit at specified index
   */
  public Visit getVisit(int index) {
    return visits.getVisits().get(index);
  }

  /**
   * return index of specified visit
   */
  public int getIndex(Visit visit) {
    return visits.getVisits().indexOf(visit);
  }

  /**
   * return hash representation of pacient
   */
  public String getHashName() {
    return hashName;
  }

  /**
   * set firstname
   */
  public void setFirstname(String name) {
    this.firstname = name;
  }

  /**
   * return firstname
   * @return String
   */
  public String getFirstname() {
    return firstname;
  }

  /**
   * set surname
   */
  public void setSurname(String name) {
    this.surname = name;
  }

  /**
   * set right-handed person
   */
  public void setRightHanded(boolean rhp) {
    this.rhp = rhp;
  }

  /**
   * return surname
   */
  public String getSurname() {
    return surname;
  }

  /**
   * set born date
   */
  public void setBornDate(Date date) {
    this.born = date;
  }

  /**
   * return date of born
   */
  public Date getBornDate() {
    return born;
  }

  /**
   * return list of visits
   */
  public List<Visit> getVisits() {
    return visits.getVisits();
  }

  /**
   * store pacient on disk
   */
  public void save() {
    if ( (surname.trim().compareTo("") != 0) ||
        (firstname.trim().compareTo("") != 0)) {
      database.addPacient(this);
    }
  }

  /**
   * add new visit to pacient
   */
  public void addVisit(Visit visit) {
    if (!visits.getVisits().contains(visit)) {
      visits.getVisits().add(visit);
      this.save();
    }
    visits.sort();
  }

  /**
   * return last visit of pacient
   */
  public Visit getLastVisit() {
    Visit res = null;
    if (visits.getVisits().size() > 0) {
      res = visits.getVisits().get(0);
      Date last = visits.getVisits().get(0).getVisitDate();
      for (Visit v : visits.getVisits()) {
        if (v.getVisitDate().compareTo(last) > 0) {
          res = v;
        }
      }
    }
    return res;
  }

  /**
   * if already exist visit with the same time, old visit is returned, otherwise new
   */
  public Visit visitExists(Visit visit) {
    int i = this.getIndex(visit);
    if (i > -1) {
      return this.getVisit(i);
    }
    return visit;
  }

  /**
   * text representation of pacient: surname + firstname
   */
  public String toString() {
    return surname + " " + firstname;
  }

  /**
   * test whether are objects the same
   */
  public boolean equals(Object o) {
    if (o == null) {
      return false;
    }
    if (o instanceof Pacient) {
      Pacient p = (Pacient) o;
      return this.getHashName().equals(p.getHashName());
    }
    return false;
  }

  /**
   * return hashcode from hash representation
   */
  public int hashCode() {
    return (int) (Long.parseLong(this.getHashName(), 16) %
                  Integer.MAX_VALUE);
  }

  /**
   * return parent database
   */
  public Database getDatabase() {
    return database;
  }

  /**
   * right-handed pacient
   */
  public boolean isRightHanded() {
    return rhp;
  }
}

/**
 * <p>Class representing list of visit. List is loaded after the first attempt to read it.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
class Visits {
  /**
   * parent pacient
   */
  private Pacient pacient;
  /**
   * engine
   */
  private Engine engine;
  /**
   * directory with pacient data
   */
  private File file;
  /**
   * list of visits
   */
  private List<Visit> visits = null;

  /**
   * creates new visit list for pacient
   */
  public Visits(Pacient pacient, Engine engine) {
    this.pacient = pacient;
    this.engine = engine;
    this.file = new File(engine.getConfiguration().getDatabaseRoot(),
                         pacient.getHashName());
  }

  /**
   * return list of visits
   * <p>Each xml file in pacient directory is treated as one visit.</p>
   */
  public List<Visit> getVisits() {
    if (visits == null) { // if visits were not already loaded
      // list xml files in pacient directory
      File[] list = file.listFiles(new FileFilter() {
        public boolean accept(File file) {
          String name = file.getName();
          int index = name.lastIndexOf(".");
          if ((index > 0) && name.substring(index).equals(".xml")) {
            return true;
          }
          return false;
        }
      });
      if (list != null) {
        // load this files
        visits = new ArrayList(list.length);
        for (File f : list) {
          visits.add(new Visit(engine, pacient, f));
        }
      }
      else {
        // there are no visits yet
        visits = new ArrayList(0);
      }
      this.sort();
    }
    return visits;
  }

  /**
   * sort visits by visit date
   */
  public void sort() {
    Collections.sort(visits, new Comparator<Visit> () {
      public int compare(Visit v1, Visit v2) {
        return v1.getVisitDate().compareTo(v2.getVisitDate());
      }
    });
  }
}
