/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.database;

import java.util.*;
import java.text.*;
import java.io.*;
import org.jdom.*;

/**
 * <p>Class representing one visit at doctor for pacient.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class Visit {
  /**
   * date of visit
   */
  private Date date = null;
  /**
   * pacient whom visit belongs to
   */
  private Pacient pacient = null;
  /**
   * engine
   */
  private Engine engine = null;
  /**
   * list of captured data
   */
  private List<Data> data = null;
  private TestList tests = null;
  /**
   * comment for visit
   */
  private String memo = "";
  /**
   * diagnose for visit
   */
  private String diagnose = "";
  /**
   * validity of diagnose
   */
  private boolean valid = false;
  /**
   * rest tremor in left hand
   */
  int restTremorLeft = 0;
  /**
   * rest tremor in right hand
   */
  int restTremorRight = 0;
  /**
   * postural tremor in left hand
   */
  int posturalTremorLeft = 0;
  /**
   * postural tremor in right hand
   */
  int posturalTremorRight = 0;
  /**
   * action tremor in left hand
   */
  int actionTremorLeft = 0;
  /**
   * action tremor in right hand
   */
  int actionTremorRight = 0;
  /**
   * opening left hand
   */
  int openingLeft = 0;
  /**
   * opening right hand
   */
  int openingRight = 0;
  /**
   * tapping with left hand
   */
  int tappingLeft = 0;
  /**
   * tapping with right hand
   */
  int tappingRight = 0;
  /**
   * format for visit's stored file
   */
  public static final String fileFormat = "yyyy_MM_dd_HH_mm";
  /**
   * name of visit tag (root element)
   */
  public static final String visitTag = "visit";
  /**
   * name of comment tag
   */
  public static final String memoTag = "memo";
  /**
   * name of diagnose tag
   */
  public static final String diagnoseTag = "diagnose";
  /**
   * name of visit's valid diagnose attribute
   */
  public static final String validAttr = "valid";
  /**
   * name of visit's left-hand rest tremor attribute
   */
  public static final String restLeftAttr = "restLeft";
  /**
   * name of visit's right-hand rest tremor attribute
   */
  public static final String restRightAttr = "restRight";
  /**
   * name of visit's left-hand rest postural attribute
   */
  public static final String postLeftAttr = "postLeft";
  /**
   * name of visit's right-hand rest postural attribute
   */
  public static final String postRightAttr = "postRight";
  /**
   * name of visit's left-hand rest action attribute
   */
  public static final String actionLeftAttr = "actionLeft";
  /**
   * name of visit's right-hand rest action attribute
   */
  public static final String actionRightAttr = "actionRight";
  /**
   * name of visit's left-hand opening attribute
   */
  public static final String openLeftAttr = "openLeft";
  /**
   * name of visit's right-hand opening attribute
   */
  public static final String openRightAttr = "openRight";
  /**
   * name of visit's left-hand tapping attribute
   */
  public static final String tapLeftAttr = "tapLeft";
  /**
   * name of visit's right-hand tapping attribute
   */
  public static final String tapRightAttr = "tapRight";

  /**
   * create new visit for pacient from last visit
   * <p>If last visit exists, its information are used for creating new visit.</p>
   */
  public Visit(Engine engine, Pacient pacient, Visit visit) {
    this(engine, pacient);
    this.date = new Date();
    this.data = new ArrayList(0);
    if(visit != null) {
      this.memo = visit.getMemo();
      this.diagnose = visit.getDiagnose();
      this.valid = visit.isValid();
      this.restTremorLeft = visit.getRestTremorLeft();
      this.restTremorRight = visit.getRestTremorRight();
      this.posturalTremorLeft = visit.getPosturalTremorLeft();
      this.posturalTremorRight = visit.getPosturalTremorRight();
      this.actionTremorLeft = visit.getActionTremorLeft();
      this.actionTremorRight = visit.getActionTremorRight();
      this.openingLeft = visit.getOpeningLeft();
      this.openingRight = visit.getOpeningRight();
      this.tappingLeft = visit.getTappingLeft();
      this.tappingRight = visit.getTappingRight();
    }
  }

  /**
   * loads visit for pacient from file
   * <p>Each visit XML file contains information about one visit at doctor and about
   * data captured at this visit with filenames of archivess with these tablet data.</p>
   */
  public Visit(Engine engine, Pacient pacient, File file) {
    this(engine, pacient);
    // create date of visit from filename
    DateFormat df = new SimpleDateFormat("'" + pacient.getHashName() + "_'" + fileFormat + "'.xml'");
    try {
      this.date = df.parse(file.getName());
    } catch(ParseException e) {
      System.err.println("Chyba parsovani dat");
    }
    // load from xml
    try {
      Document doc = engine.getXMLBuilder().build(file);
      this.loadFromElement(doc.getRootElement());
    }
    catch (JDOMException e) {
      System.err.println("Chyba parsovani souboru databaze.");
      System.err.println(e.getMessage());
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  /**
   * common inicialization
   */
  private Visit(Engine engine, Pacient pacient) {
    this.pacient = pacient;
    this.engine = engine;
    tests = new TestList(this);
  }

  /**
   * load information from XML element
   */
  private void loadFromElement(Element root) {
    valid = Integer.parseInt(root.getAttributeValue(validAttr)) == 1 ? true : false;
    restTremorLeft = Integer.parseInt(root.getAttributeValue(restLeftAttr));
    restTremorRight = Integer.parseInt(root.getAttributeValue(restRightAttr));
    posturalTremorLeft = Integer.parseInt(root.getAttributeValue(postLeftAttr));
    posturalTremorRight = Integer.parseInt(root.getAttributeValue(postRightAttr));
    actionTremorLeft = Integer.parseInt(root.getAttributeValue(actionLeftAttr));
    actionTremorRight = Integer.parseInt(root.getAttributeValue(actionRightAttr));
    openingLeft = Integer.parseInt(root.getAttributeValue(openLeftAttr));
    openingRight = Integer.parseInt(root.getAttributeValue(openRightAttr));
    tappingLeft = Integer.parseInt(root.getAttributeValue(tapLeftAttr));
    tappingRight = Integer.parseInt(root.getAttributeValue(tapRightAttr));
    memo = root.getChild(memoTag).getText();
    diagnose = root.getChild(diagnoseTag).getText();

    // load data
    List<Element> list = root.getChildren(Data.dataTag);
    this.data = new ArrayList(list.size());
    for(Element e: list) {
      Data d = new Data(this, e);
      data.add(d);
      tests.addHist(d.getDescription());
    }
  }

  /**
   * return XML representation of visit
   * <p>Tremor measure, comment, diagnose etc. Also list of data captured at this visit.<p>
   */
  public Element getElement() {
    Element root = new Element(visitTag);
    root.setAttribute(validAttr, valid ? "1" : "0");
    root.setAttribute(restLeftAttr, Integer.toString(restTremorLeft));
    root.setAttribute(restRightAttr, Integer.toString(restTremorRight));
    root.setAttribute(postLeftAttr, Integer.toString(posturalTremorLeft));
    root.setAttribute(postRightAttr, Integer.toString(posturalTremorRight));
    root.setAttribute(actionLeftAttr, Integer.toString(actionTremorLeft));
    root.setAttribute(actionRightAttr, Integer.toString(actionTremorRight));
    root.setAttribute(openLeftAttr, Integer.toString(openingLeft));
    root.setAttribute(openRightAttr, Integer.toString(openingRight));
    root.setAttribute(tapLeftAttr, Integer.toString(tappingLeft));
    root.setAttribute(tapRightAttr, Integer.toString(tappingRight));
    Element diag = new Element(diagnoseTag);
    diag.setText(diagnose);
    root.addContent(diag);
    Element m = new Element(memoTag);
    m.setText(memo);
    root.addContent(m);
    for(Data d: data) {
      root.addContent(d.getElement());
    }
    return root;
  }

  /**
   * return text representation of visit = date: DD. MM. YYYY, HH:MM
   */
  public String toString() {
    SimpleDateFormat df = new SimpleDateFormat("dd. MM. yyyy, HH:mm");
    return df.format(date);
  }

  /**
   * test of equality of two visits
   */
  public boolean equals(Object o) {
    if(o == null) return false;
    if(o instanceof Visit) {
      Visit v = (Visit) o;
      return this.getPacient().equals(v.getPacient()) && this.toString().equals(v.toString());
    }
    return false;
  }

  /**
   * return parent pacient
   */
  public Pacient getPacient() {
    return pacient;
  }

  /**
   * return list of captured data
   */
  public List<Data> getData() {
    return data;
  }

  /**
   * return date of visit
   */
  public Date getVisitDate() {
    return date;
  }

  /**
   * return comment
   */
  public String getMemo() {
    return memo;
  }

  /**
   * return diagnose
   */
  public String getDiagnose() {
    return diagnose;
  }

  /**
   * return validity of diagnose
   */
  public boolean isValid() {
    return valid;
  }

  /**
   * set new comment
   */
  public void setMemo(String memo) {
    this.memo = memo;
  }

  /**
   * set new diagnose
   */
  public void setDiagnose(String diagnose) {
    this.diagnose = diagnose;
  }

  /**
   * set validity of diagnose
   */
  public void setValid(boolean valid) {
    this.valid = valid;
  }

  /**
   * set date of visit
   */
  public void setVisitDate(Date date) {
    this.date = date;
  }

  /**
   * return left-hand rest tremor
   */
  public int getRestTremorLeft() {
    return restTremorLeft;
  }

  /**
   * return right-hand rest tremor
   */
  public int getRestTremorRight() {
    return restTremorRight;
  }

  /**
   * return left-hand postural tremor
   */
  public int getPosturalTremorLeft() {
    return posturalTremorLeft;
  }

  /**
   * return right-hand postural tremor
   */
  public int getPosturalTremorRight() {
    return posturalTremorRight;
  }

  /**
   * return left-hand action tremor
   */
  public int getActionTremorLeft() {
    return actionTremorLeft;
  }

  /**
   * return right-hand action tremor
   */
  public int getActionTremorRight() {
    return actionTremorRight;
  }

  /**
   * return left-hand opening
   */
  public int getOpeningLeft() {
    return openingLeft;
  }

  /**
   * return right-hand opening
   */
  public int getOpeningRight() {
    return openingRight;
  }

  /**
   * return left-hand tapping
   */
  public int getTappingLeft() {
    return tappingLeft;
  }

  /**
   * return right-hand tapping
   */
  public int getTappingRight() {
    return tappingRight;
  }

  /**
   * set left-hand rest tremor
   */
  public void setRestTremorLeft(int value) {
    restTremorLeft = value;
  }

  /**
   * set right-hand rest tremor
   */
  public void setRestTremorRight(int value) {
    restTremorRight = value;
  }

  /**
   * set left-hand postural tremor
   */
  public void setPosturalTremorLeft(int value) {
    posturalTremorLeft = value;
  }

  /**
   * set right-hand postural tremor
   */
  public void setPosturalTremorRight(int value) {
    posturalTremorRight = value;
  }

  /**
   * set left-hand action tremor
   */
  public void setActionTremorLeft(int value) {
    actionTremorLeft = value;
  }

  /**
   * set right-hand action tremor
   */
  public void setActionTremorRight(int value) {
    actionTremorRight = value;
  }

  /**
   * set left-hand opening
   */
  public void setOpeningLeft(int value) {
    openingLeft = value;
  }

  /**
   * set right-hand opening
   */
  public void setOpeningRight(int value) {
    openingRight = value;
  }

  /**
   * set left-hand tapping
   */
  public void setTappingLeft(int value) {
    tappingLeft = value;
  }

  /**
   * set right-hand tapping
   */
  public void setTappingRight(int value) {
    tappingRight = value;
  }

  /**
   * store visit to disk
   */
  public void save() {
    File dir = new File(engine.getConfiguration().getDatabaseRoot(), pacient.getHashName());
    if(!dir.exists()) {
      dir.mkdir();
    }
    pacient.addVisit(this); // update pacient
    Document doc = new Document(getElement());

    OutputStream os = null;
    try {
      os = new FileOutputStream(getFile());
      engine.getXMLOutputer().output(doc, os);
    }
    catch (IOException e) {
      System.err.println(e);
    } finally {
      if(os != null) {
        try {
          os.close();
        } catch (IOException e) {}
      }
    }
  }

  /**
   * return file where is visit stored on disk
   */
  public File getFile() {
    File dir = new File(engine.getConfiguration().getDatabaseRoot(), pacient.getHashName());
    DateFormat df = new SimpleDateFormat(fileFormat);
    return new File(dir, pacient.getHashName() + "_" + df.format(date) + ".xml");
  }

  /**
   * add captured data to list
   */
  public void addData(Data data) {
    this.data.add(data);
    tests.addHist(data.getDescription());
  }

  /**
   * return histogram of captured data types
   */
  public TestList getTestList() {
    return tests;
  }
}
